@@ -1,5 +1,10 @@
 from utilities.choices import ChoiceSet
 
+#
+# Assets
+#
+
+
 class AssetStatusChoices(ChoiceSet):
     key = 'Asset.status'
 
@@ -15,9 +20,15 @@ class HardwareKindChoices(ChoiceSet):
         ('device', 'Device'),
         ('module', 'Module'),
         ('inventoryitem', 'Inventory Item'),
+        ('rack', 'Rack'),
     ]
 
 
+#
+# Deliveries
+#
+
+
 class PurchaseStatusChoices(ChoiceSet):
     key = 'Purchase.status'
 
