@@ -1,13 +1,21 @@
-from django.db.models.functions import Coalesce
 import django_tables2 as tables
+from django.db.models.functions import Coalesce
+from django.utils.translation import gettext_lazy as _
 
-from netbox.tables import columns, NetBoxTable
+from dcim.tables import DeviceTypeTable, ModuleTypeTable, RackTypeTable
+from netbox.tables import NetBoxTable, columns
 from tenancy.tables import ContactsColumnMixin
-from .models import Asset, Delivery, InventoryItemType, InventoryItemGroup, Purchase, Supplier
+from utilities.tables import register_table_column
+
+from .models import *
 from .template_content import WARRANTY_PROGRESSBAR
 
 __all__ = (
     'AssetTable',
+    'AuditFlowPageAssignmentTable',
+    'AuditFlowPageTable',
+    'AuditFlowTable',
+    'AuditTrailTable',
     'SupplierTable',
     'PurchaseTable',
     'DeliveryTable',
@@ -16,6 +24,93 @@ __all__ = (
 )
 
 
+#
+# Assets
+#
+
+
+class InventoryItemGroupTable(NetBoxTable):
+    name = columns.MPTTColumn(
+        linkify=True,
+    )
+    asset_count = columns.LinkedCountColumn(
+        viewname='plugins:netbox_inventory:asset_list',
+        url_params={'inventoryitem_group_id': 'pk'},
+        verbose_name='Assets',
+    )
+    inventoryitem_type_count = columns.LinkedCountColumn(
+        viewname='plugins:netbox_inventory:inventoryitemtype_list',
+        url_params={'inventoryitem_group_id': 'pk'},
+        verbose_name='Inventory Item Types',
+    )
+    comments = columns.MarkdownColumn()
+    tags = columns.TagColumn()
+
+    class Meta(NetBoxTable.Meta):
+        model = InventoryItemGroup
+        fields = (
+            'pk',
+            'id',
+            'name',
+            'description',
+            'comments',
+            'tags',
+            'created',
+            'last_updated',
+            'actions',
+            'asset_count',
+            'inventoryitem_type_count',
+        )
+        default_columns = (
+            'name',
+            'asset_count',
+            'inventoryitem_type_count',
+        )
+
+
+class InventoryItemTypeTable(NetBoxTable):
+    manufacturer = tables.Column(
+        linkify=True,
+    )
+    model = tables.Column(
+        linkify=True,
+    )
+    inventoryitem_group = tables.Column(
+        linkify=True,
+    )
+    asset_count = columns.LinkedCountColumn(
+        viewname='plugins:netbox_inventory:asset_list',
+        url_params={'inventoryitem_type_id': 'pk'},
+        verbose_name='Assets',
+    )
+    comments = columns.MarkdownColumn()
+    tags = columns.TagColumn()
+
+    class Meta(NetBoxTable.Meta):
+        model = InventoryItemType
+        fields = (
+            'pk',
+            'id',
+            'manufacturer',
+            'model',
+            'slug',
+            'part_number',
+            'inventoryitem_group',
+            'description',
+            'comments',
+            'tags',
+            'created',
+            'last_updated',
+            'actions',
+            'asset_count',
+        )
+        default_columns = (
+            'manufacturer',
+            'model',
+            'asset_count',
+        )
+
+
 class AssetTable(NetBoxTable):
     name = tables.Column(
         linkify=True,
@@ -45,6 +140,11 @@ class AssetTable(NetBoxTable):
         linkify=True,
         order_by=('device', 'module'),
     )
+    hardware_role = tables.Column(
+        accessor=columns.Accessor('hardware__role'),
+        linkify=True,
+        verbose_name='Hardware Role',
+    )
     installed_site = tables.Column(
         linkify=True,
         verbose_name='Installed Site',
@@ -90,12 +190,21 @@ class AssetTable(NetBoxTable):
     delivery_date = columns.DateColumn(
         accessor='delivery__date',
         verbose_name='Delivery Date',
-
+    )
+    current_site = tables.Column(
+        linkify=True,
+        verbose_name='Current Site',
+        orderable=False,
+    )
+    current_location = tables.Column(
+        linkify=True,
+        verbose_name='Current Location',
+        orderable=False,
     )
     warranty_progress = columns.TemplateColumn(
         template_code=WARRANTY_PROGRESSBAR,
         order_by='warranty_end',
-        #orderable=False,
+        # orderable=False,
         verbose_name='Warranty remaining',
     )
     comments = columns.MarkdownColumn()
@@ -123,6 +232,7 @@ class AssetTable(NetBoxTable):
                 'device_type__manufacturer',
                 'module_type__manufacturer',
                 'inventoryitem_type__manufacturer',
+                'rack_type__manufacturer',
             )
         ).order_by(
             ('-' if is_descending else '') + 'manufacturer',
@@ -134,7 +244,10 @@ class AssetTable(NetBoxTable):
         queryset, _ = self.order_manufacturer(queryset, is_descending)
         queryset = queryset.annotate(
             model=Coalesce(
-                'device_type__model', 'module_type__model', 'inventoryitem_type__model'
+                'device_type__model',
+                'module_type__model',
+                'inventoryitem_type__model',
+                'rack_type__model',
             )
         ).order_by(
             ('-' if is_descending else '') + 'manufacturer',
@@ -146,7 +259,10 @@ class AssetTable(NetBoxTable):
     def order_hardware(self, queryset, is_descending):
         queryset = queryset.annotate(
             hw=Coalesce(
-                'device__name', 'module__device__name', 'inventoryitem__device__name'
+                'device__name',
+                'module__device__name',
+                'inventoryitem__device__name',
+                'rack__name',
             )
         ).order_by(
             ('-' if is_descending else '') + 'hw',
@@ -154,21 +270,43 @@ class AssetTable(NetBoxTable):
             ('-' if is_descending else '') + 'serial',
         )
         return (queryset, True)
-    
+
+    def order_hardware_role(self, queryset, is_descending):
+        queryset = queryset.annotate(
+            role_name=Coalesce(
+                'device__role__name',
+                'inventoryitem__role__name',
+                'rack__role__name',
+            )
+        ).order_by(
+            ('-' if is_descending else '') + 'role_name',
+            ('-' if is_descending else '') + 'serial',
+        )
+        return (queryset, True)
+
     def _order_annotate_installed(self, queryset):
         return queryset.annotate(
             site_name=Coalesce(
-                'device__site__name', 'module__device__site__name', 'inventoryitem__device__site__name'
+                'device__site__name',
+                'module__device__site__name',
+                'inventoryitem__device__site__name',
+                'rack__site__name',
             ),
             location_name=Coalesce(
-                'device__location__name', 'module__device__location__name', 'inventoryitem__device__location__name'
+                'device__location__name',
+                'module__device__location__name',
+                'inventoryitem__device__location__name',
+                'rack__location__name',
             ),
             rack_name=Coalesce(
-                'device__rack__name', 'module__device__rack__name', 'inventoryitem__device__rack__name'
+                'device__rack__name',
+                'module__device__rack__name',
+                'inventoryitem__device__rack__name',
+                'rack__name',
             ),
             device_name=Coalesce(
                 'device__name', 'module__device__name', 'inventoryitem__device__name'
-            )
+            ),
         )
 
     def order_installed_site(self, queryset, is_descending):
@@ -223,13 +361,17 @@ class AssetTable(NetBoxTable):
             'hardware_type',
             'inventoryitem_group',
             'hardware',
+            'hardware_role',
             'installed_site',
             'installed_location',
             'installed_rack',
             'installed_device',
             'tenant',
             'contact',
+            'storage_site',
             'storage_location',
+            'current_site',
+            'current_location',
             'owner',
             'supplier',
             'purchase',
@@ -239,6 +381,7 @@ class AssetTable(NetBoxTable):
             'warranty_start',
             'warranty_end',
             'warranty_progress',
+            'description',
             'comments',
             'tags',
             'created',
@@ -259,6 +402,11 @@ class AssetTable(NetBoxTable):
         )
 
 
+#
+# Deliveries
+#
+
+
 class SupplierTable(ContactsColumnMixin, NetBoxTable):
     name = tables.Column(
         linkify=True,
@@ -408,81 +556,192 @@ class DeliveryTable(NetBoxTable):
         )
 
 
-class InventoryItemTypeTable(NetBoxTable):
-    manufacturer = tables.Column(
+#
+# Audit
+#
+
+
+class BaseFlowTable(NetBoxTable):
+    """
+    Internal base table class for audit flow models.
+    """
+
+    name = tables.Column(
         linkify=True,
     )
-    model = tables.Column(
-        linkify=True,
+    object_type = columns.ContentTypeColumn(
+        verbose_name=_('Object Type'),
     )
-    inventoryitem_group = tables.Column(
-        linkify=True,
-    )
-    asset_count = columns.LinkedCountColumn(
-        viewname='plugins:netbox_inventory:asset_list',
-        url_params={'inventoryitem_type_id': 'pk'},
-        verbose_name='Assets',
-    )
-    comments = columns.MarkdownColumn()
-    tags = columns.TagColumn()
 
     class Meta(NetBoxTable.Meta):
-        model = InventoryItemType
-        fields = (
-            'pk',
-            'id',
-            'manufacturer',
-            'model',
-            'slug',
-            'part_number',
-            'inventoryitem_group',
-            'comments',
-            'tags',
-            'created',
-            'last_updated',
-            'actions',
-            'asset_count',
-        )
-        default_columns = (
-            'manufacturer',
-            'model',
-            'asset_count',
-        )
-
-
-class InventoryItemGroupTable(NetBoxTable):
-    name = columns.MPTTColumn(
-        linkify=True,
-    )
-    asset_count = columns.LinkedCountColumn(
-        viewname='plugins:netbox_inventory:asset_list',
-        url_params={'inventoryitem_group_id': 'pk'},
-        verbose_name='Assets',
-    )
-    inventoryitem_type_count = columns.LinkedCountColumn(
-        viewname='plugins:netbox_inventory:inventoryitemtype_list',
-        url_params={'inventoryitem_group_id': 'pk'},
-        verbose_name='Inventory Item Types',
-    )
-    comments = columns.MarkdownColumn()
-    tags = columns.TagColumn()
-
-    class Meta(NetBoxTable.Meta):
-        model = InventoryItemGroup
         fields = (
             'pk',
             'id',
             'name',
+            'description',
+            'object_type',
+            'object_filter',
             'comments',
-            'tags',
-            'created',
-            'last_updated',
             'actions',
-            'asset_count',
-            'inventoryitem_type_count',
         )
         default_columns = (
             'name',
-            'asset_count',
-            'inventoryitem_type_count',
+            'object_type',
         )
+
+
+class AuditFlowPageTable(BaseFlowTable):
+    class Meta(BaseFlowTable.Meta):
+        model = AuditFlowPage
+
+
+class AuditFlowTable(BaseFlowTable):
+    enabled = columns.BooleanColumn()
+
+    class Meta(BaseFlowTable.Meta):
+        model = AuditFlow
+        fields = BaseFlowTable.Meta.fields + ('enabled',)
+        default_columns = BaseFlowTable.Meta.default_columns + ('enabled',)
+
+
+class AuditFlowPageAssignmentTable(NetBoxTable):
+    flow = tables.Column(
+        linkify=True,
+    )
+    page = tables.Column(
+        linkify=True,
+    )
+
+    actions = columns.ActionsColumn(
+        actions=(
+            'edit',
+            'delete',
+        ),
+    )
+
+    class Meta(NetBoxTable.Meta):
+        model = AuditFlowPageAssignment
+        fields = (
+            'pk',
+            'id',
+            'flow',
+            'page',
+            'weight',
+            'actions',
+        )
+        default_columns = (
+            'flow',
+            'page',
+            'weight',
+        )
+
+
+class AuditTrailSourceTable(NetBoxTable):
+    name = tables.Column(
+        linkify=True,
+    )
+
+    class Meta(NetBoxTable.Meta):
+        model = AuditTrailSource
+        fields = (
+            'pk',
+            'id',
+            'name',
+            'description',
+            'comments',
+            'actions',
+        )
+        default_columns = ('name',)
+
+
+class AuditTrailTable(NetBoxTable):
+    object_type = columns.ContentTypeColumn(
+        verbose_name=_('Object Type'),
+    )
+    object = tables.Column(
+        verbose_name=_('Object'),
+        linkify=True,
+        orderable=False,
+    )
+    source = tables.Column(
+        linkify=True,
+    )
+    created = columns.DateTimeColumn(
+        verbose_name=_('Time'),
+        timespec='minutes',
+    )
+    actions = columns.ActionsColumn(
+        actions=('delete',),
+    )
+
+    # Access the audit user via the first associated object change.
+    auditor_user = tables.Column(
+        accessor=tables.A('object_changes__first__user'),
+        verbose_name=_('Auditor'),
+        linkify=True,
+        order_by=('object_change__user'),
+    )
+    auditor_full_name = tables.Column(
+        accessor=tables.A('object_changes__first__user__get_full_name'),
+        verbose_name=_('Auditor Name'),
+        linkify=True,
+        orderable=False,
+    )
+
+    class Meta(NetBoxTable.Meta):
+        model = AuditTrail
+        fields = (
+            'pk',
+            'id',
+            'object_type',
+            'object',
+            'auditor_user',
+            'auditor_full_name',
+            'source',
+            'created',
+            'last_changed',
+            'actions',
+        )
+        default_columns = (
+            'pk',
+            'created',
+            'object_type',
+            'object',
+            'auditor_user',
+            'auditor_full_name',
+            'source',
+        )
+
+
+# ========================
+# DCIM model table columns
+# ========================
+
+asset_count = columns.LinkedCountColumn(
+    viewname='plugins:netbox_inventory:asset_list',
+    url_params={'device_type_id': 'pk'},
+    verbose_name=_('Assets'),
+    accessor='assets__count',
+)
+
+register_table_column(asset_count, 'assets', DeviceTypeTable)
+
+
+asset_count = columns.LinkedCountColumn(
+    viewname='plugins:netbox_inventory:asset_list',
+    url_params={'module_type_id': 'pk'},
+    verbose_name=_('Assets'),
+    accessor='assets__count',
+)
+
+register_table_column(asset_count, 'assets', ModuleTypeTable)
+
+
+asset_count = columns.LinkedCountColumn(
+    viewname='plugins:netbox_inventory:asset_list',
+    url_params={'rack_type_id': 'pk'},
+    verbose_name=_('Assets'),
+    accessor='assets__count',
+)
+
+register_table_column(asset_count, 'assets', RackTypeTable)
