@@ -1,13 +1,13 @@
-from dcim.models import Device, Module, InventoryItem
+from dcim.models import Device, InventoryItem, Module, Rack
 from netbox.views import generic
 
 from ..forms.reassign import *
 
-
 __all__ = (
     'AssetDeviceReassignView',
     'AssetModuleReassignView',
     'AssetInventoryItemReassignView',
+    'AssetRackReassignView',
 )
 
 
@@ -27,3 +27,9 @@ class AssetInventoryItemReassignView(generic.ObjectEditView):
     queryset = InventoryItem.objects.all()
     template_name = 'netbox_inventory/asset_reassign.html'
     form = AssetInventoryItemReassignForm
+
+
+class AssetRackReassignView(generic.ObjectEditView):
+    queryset = Rack.objects.all()
+    template_name = 'netbox_inventory/asset_reassign.html'
+    form = AssetRackReassignForm
